@@ -118,6 +118,7 @@ DEFINE_TEST(test_bad_fd)
 DEFINE_TEST(test_compat_bzip2)
 DEFINE_TEST(test_compat_cpio)
 DEFINE_TEST(test_compat_gtar)
+DEFINE_TEST(test_compat_gtar_large)
 DEFINE_TEST(test_compat_gzip)
 DEFINE_TEST(test_compat_lz4)
 DEFINE_TEST(test_compat_lzip)
@@ -222,6 +223,9 @@ DEFINE_TEST(test_read_format_7zip_lzma2_arm64)
 DEFINE_TEST(test_read_format_7zip_deflate_arm64)
 DEFINE_TEST(test_read_format_7zip_win_attrib)
 DEFINE_TEST(test_read_format_7zip_extract_second)
+DEFINE_TEST(test_read_format_7zip_lzma2_sparc)
+DEFINE_TEST(test_read_format_7zip_zstd_sparc)
+DEFINE_TEST(test_read_format_7zip_deflate_powerpc)
 DEFINE_TEST(test_read_format_7zip_encryption_data)
 DEFINE_TEST(test_read_format_7zip_encryption_header)
 DEFINE_TEST(test_read_format_7zip_encryption_partially)
@@ -275,6 +279,7 @@ DEFINE_TEST(test_read_format_gtar_filename_KOI8R_CP1251)
 DEFINE_TEST(test_read_format_gtar_gz)
 DEFINE_TEST(test_read_format_gtar_lzma)
 DEFINE_TEST(test_read_format_gtar_sparse)
+DEFINE_TEST(test_read_format_gtar_sparse_length)
 DEFINE_TEST(test_read_format_gtar_sparse_skip_entry)
 DEFINE_TEST(test_read_format_huge_rpm)
 DEFINE_TEST(test_read_format_iso_Z)
