@@ -24,6 +24,10 @@
  */
 #include "test.h"
 
+#if HAVE_LZMA_H
+#include <lzma.h>
+#endif
+
 #if defined(_WIN32) && !defined(__CYGWIN__)
 #define	close		_close
 #define	open		_open
@@ -929,6 +933,10 @@ DEFINE_TEST(test_read_format_7zip_zstd)
 	if (ARCHIVE_OK != archive_read_support_filter_zstd(a)) {
 		skipping(
 		    "7zip:zstd decoding is not supported on this platform");
+	} else if (ARCHIVE_OK != archive_read_support_filter_xz(a)) {
+		// The directory header entries in the test file uses lzma.
+		skipping(
+		    "7zip:lzma decoding is not supported on this platform");
 	} else {
 		test_extract_all_files_zstd("test_read_format_7zip_zstd.7z");
 	}
@@ -946,6 +954,10 @@ DEFINE_TEST(test_read_format_7zip_zstd_solid)
 	if (ARCHIVE_OK != archive_read_support_filter_zstd(a)) {
 		skipping(
 		    "7zip:zstd decoding is not supported on this platform");
+	} else if (ARCHIVE_OK != archive_read_support_filter_xz(a)) {
+		// The directory header entries in the test file uses lzma.
+		skipping(
+		    "7zip:lzma decoding is not supported on this platform");
 	} else {
 		test_extract_all_files_zstd("test_read_format_7zip_solid_zstd.7z");
 	}
@@ -1050,6 +1062,7 @@ test_arm_filter(const char *refname)
 
 	extract_reference_file(refname);
 
+	assertEqualInt(ARCHIVE_OK, archive_read_free(a));
 	assert((a = archive_read_new()) != NULL);
 	assertEqualIntA(a, ARCHIVE_OK, archive_read_support_filter_all(a));
 	assertEqualIntA(a, ARCHIVE_OK, archive_read_support_format_all(a));
@@ -1123,6 +1136,7 @@ test_arm64_filter(const char *refname)
 
 	extract_reference_file(refname);
 
+	assertEqualInt(ARCHIVE_OK, archive_read_free(a));
 	assert((a = archive_read_new()) != NULL);
 	assertEqualIntA(a, ARCHIVE_OK, archive_read_support_filter_all(a));
 	assertEqualIntA(a, ARCHIVE_OK, archive_read_support_format_all(a));
@@ -1148,7 +1162,7 @@ test_arm64_filter(const char *refname)
 
 DEFINE_TEST(test_read_format_7zip_lzma2_arm64)
 {
-#ifdef HAVE_LZMA_FILTER_ARM64
+#ifdef LZMA_FILTER_ARM64
 	struct archive *a;
 
 	assert((a = archive_read_new()) != NULL);
@@ -1255,5 +1269,177 @@ DEFINE_TEST(test_read_format_7zip_win_attrib)
 	assertEqualString("system", archive_entry_fflags_text(ae));
 
 
+	assertEqualInt(ARCHIVE_OK, archive_read_free(a));
+}
+
+DEFINE_TEST(test_read_format_7zip_extract_second)
+{
+	struct archive *a;
+	char buffer[256];
+
+	assert((a = archive_read_new()) != NULL);
+
+	if (ARCHIVE_OK != archive_read_support_filter_lzma(a)) {
+		skipping(
+		    "7zip:lzma decoding is not supported on this platform");
+		assertEqualInt(ARCHIVE_OK, archive_read_free(a));
+		return;
+	}
+
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_support_format_all(a));
+
+	/*
+	 * The test archive has two files: first.txt which is a 65,536 file (the
+	 * size of the uncompressed buffer), and second.txt which has contents
+	 * we will validate. This test ensures we can skip first.txt and still
+	 * be able to read the contents of second.txt
+	 */
+	const char *refname = "test_read_format_7zip_extract_second.7z";
+	extract_reference_file(refname);
+
+	assertEqualIntA(a, ARCHIVE_OK,
+		archive_read_open_filename(a, refname, 10240));
+
+	struct archive_entry *ae;
+
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_next_header(a, &ae));
+	assertEqualString("first.txt", archive_entry_pathname(ae));
+
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_next_header(a, &ae));
+	assertEqualString("second.txt", archive_entry_pathname(ae));
+
+	assertEqualInt(23, archive_read_data(a, buffer, sizeof(buffer)));
+	assertEqualMem("This is from second.txt", buffer, 23);
+
+	assertEqualInt(ARCHIVE_OK, archive_read_free(a));
+}
+
+static void
+test_sparc_filter(const char *refname)
+{
+	struct archive *a;
+	struct archive_entry *ae;
+	size_t expected_entry_size = 1053016;
+	char *buff = malloc(expected_entry_size);
+	uint32_t computed_crc = 0;
+	uint32_t expected_crc = 0x6b5b364d;
+
+	assert((a = archive_read_new()) != NULL);
+
+	extract_reference_file(refname);
+
+	assertEqualInt(ARCHIVE_OK, archive_read_free(a));
+	assert((a = archive_read_new()) != NULL);
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_support_filter_all(a));
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_support_format_all(a));
+
+	assertEqualIntA(a, ARCHIVE_OK,
+		archive_read_open_filename(a, refname, 10240));
+
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_next_header(a, &ae));
+	assertEqualInt((AE_IFREG | 0775), archive_entry_mode(ae));
+	assertEqualString("hw-sparc64", archive_entry_pathname(ae));
+	assertEqualInt(expected_entry_size, archive_entry_size(ae));
+	assertEqualInt(expected_entry_size, archive_read_data(a, buff, expected_entry_size));
+
+	computed_crc = crc32(computed_crc, buff, expected_entry_size);
+	assertEqualInt(computed_crc, expected_crc);
+
+	assertEqualInt(1, archive_file_count(a));
+
+	assertEqualIntA(a, ARCHIVE_EOF, archive_read_next_header(a, &ae));
+
+	assertEqualInt(ARCHIVE_OK, archive_read_close(a));
+	assertEqualInt(ARCHIVE_OK, archive_read_free(a));
+
+	free(buff);
+}
+
+DEFINE_TEST(test_read_format_7zip_lzma2_sparc)
+{
+	struct archive *a;
+
+	assert((a = archive_read_new()) != NULL);
+
+	if (ARCHIVE_OK != archive_read_support_filter_lzma(a)) {
+		skipping(
+		    "7zip:lzma decoding is not supported on this platform");
+	} else {
+		test_sparc_filter("test_read_format_7zip_lzma2_sparc.7z");
+	}
+
+	assertEqualInt(ARCHIVE_OK, archive_read_free(a));
+}
+
+DEFINE_TEST(test_read_format_7zip_zstd_sparc)
+{
+	struct archive *a;
+
+	assert((a = archive_read_new()) != NULL);
+
+	if (ARCHIVE_OK != archive_read_support_filter_zstd(a)) {
+		skipping(
+		    "7zip:zstd decoding is not supported on this platform");
+	} else {
+		test_sparc_filter("test_read_format_7zip_zstd_sparc.7z");
+	}
+
+	assertEqualInt(ARCHIVE_OK, archive_read_free(a));
+}
+
+static void
+test_powerpc_filter(const char *refname)
+{
+	struct archive *a;
+	struct archive_entry *ae;
+	size_t expected_entry_size = 68340;
+	char *buff = malloc(expected_entry_size);
+	uint32_t computed_crc = 0;
+	uint32_t expected_crc = 0x71fb03c9;
+
+	assert((a = archive_read_new()) != NULL);
+
+	extract_reference_file(refname);
+
+	assertEqualInt(ARCHIVE_OK, archive_read_free(a));
+	assert((a = archive_read_new()) != NULL);
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_support_filter_all(a));
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_support_format_all(a));
+
+	assertEqualIntA(a, ARCHIVE_OK,
+		archive_read_open_filename(a, refname, 10240));
+
+	assertEqualIntA(a, ARCHIVE_OK, archive_read_next_header(a, &ae));
+	assertEqualInt((AE_IFREG | 0775), archive_entry_mode(ae));
+	assertEqualString("hw-powerpc", archive_entry_pathname(ae));
+	assertEqualInt(expected_entry_size, archive_entry_size(ae));
+	assertEqualInt(expected_entry_size, archive_read_data(a, buff, expected_entry_size));
+
+	computed_crc = crc32(computed_crc, buff, expected_entry_size);
+	assertEqualInt(computed_crc, expected_crc);
+
+	assertEqualInt(1, archive_file_count(a));
+
+	assertEqualIntA(a, ARCHIVE_EOF, archive_read_next_header(a, &ae));
+
+	assertEqualInt(ARCHIVE_OK, archive_read_close(a));
+	assertEqualInt(ARCHIVE_OK, archive_read_free(a));
+
+	free(buff);
+}
+
+DEFINE_TEST(test_read_format_7zip_deflate_powerpc)
+{
+	struct archive *a;
+
+	assert((a = archive_read_new()) != NULL);
+
+	if (ARCHIVE_OK != archive_read_support_filter_gzip(a)) {
+		skipping(
+		    "7zip:deflate decoding is not supported on this platform");
+	} else {
+		test_powerpc_filter("test_read_format_7zip_deflate_powerpc.7z");
+	}
+
 	assertEqualInt(ARCHIVE_OK, archive_read_free(a));
 }
